% General class representing a non-parametric Bayesian network with
% conditional kernel density estimates as local models.
%   name   : Name identifying the network.
%   data   : (samples,dim) double array. The training data.
%   options: Matlab structure with the following fields:
%     class          : Only 'CKDE' possible right now.
%     graphEstimation: Only 'ChowLiu' possible right now.
%     graph          : (#variables,#variables) binary matrix. Optional field.
%                      If available, computation of the Chow-Liu tree is skipped and the
%                      supplied graph structure is used instead.
%     best           : The employed technique for KDE bandwidth estimation.
%                      Possible values are {'full','diagonal','isotropic'}. 
%                      Optional field with default 'full'.
%     enableAll      : Whether to use approximate log-likelihoods. 
%                      Possible values are {0,1}. Optional field with default 0.
%     totalClusters  : The total number of clusters to use for approximate log-likelihood computations.
%                      Possible values are scalar integers between 1 and samples.
%                      Only relevant if enableAll==1. Optional field with default 50.
%     names          : (#variables,1) cell array containing the names of the variables, e.g., {'A','B','C'}. 
%     structure      : (#variables,1) cell array mapping variables to data channels.
%                      For instance, {1:3,4:6,7:9} would assign columns 1:3 to variable 'A', 4:6 to
%                      variable 'B' and 7:9 to variable 'C'.
% 
% (c)2014 Andreas M. Lehrmann

classdef BayesianNetwork < handle
    
    properties
        name = 'Untitled'
        graph
        variables
        ordering
    end
    
    methods        
        function obj = BayesianNetwork(name,data,options)    
            if nargin>0
                obj.name = name;
             
                lDisp('Constructing training data...')
                obj.variables = struct('name',{},'data',{},'cpd',{});
                for i=1:length(options.names)
                    obj.variables(i).name = options.names{i};
                    obj.variables(i).channels = options.structure{i};
                    obj.variables(i).data = data(:,obj.variables(i).channels)';
                    obj.variables(i).cpd = [];
                end
                
                [distributedData{1:length(options.names)}] = deal(obj.variables.data);
                if strcmp(options.graphEstimation,'ChowLiu')
                    if isfield(options,'graph')
                        lDisp('Found graph structure...skipping computation of the Chow-Liu tree...')
                        obj.graph = options.graph;
                    else
                        lDisp('Learning graph structure...')
                        obj.graph = obj.chowLiu(distributedData,@H_KNN);
                        obj.graph(find(obj.graph))=1;
                    end
                else
                    error('Error.'); 
                end
                
                lDisp('Assigning parents...')
                for i=1:length(options.names)
                    obj.variables(i).parent = obj.getPa(i);
                end
                    
                lDisp('Sorting topology...')
                obj.ordering = obj.topSort(obj.graph);
                
                lDisp('Learning local conditional distributions...')
                if strcmp(options.class,'CKDE') 
                    obj.learnKDEN(options);
                else
                    error('Error.'); 
                end
            end
        end
    end
end